import boto3
from botocore.exceptions import ClientError
import os
import logging

class AwsCloudtils(object):
    def __init__(self, access_key, secret_key):
        self.access_key = access_key
        self.secret_key = secret_key
        self.client = boto3.client('s3',
                            aws_access_key_id=self.access_key,
                            aws_secret_access_key=self.secret_key,
                        )

    def upload_file(self, fpath, bucket, object_name=None):
        if object_name is None:
            object_name = os.path.basename(fpath)
        try:
            response = self.client.upload_file(fpath, bucket, object_name)
        except ClientError as e:
            logging.error(e)
            return False
        return True

    def download_file(self, bucket, object_name, fpath):
        try:
            self.client.download_file(bucket, object_name, fpath)
        except ClientError as e:
            logging.error(e)
            return False
        return True



import hashlib
from Crypto import Random
from Crypto.Cipher import AES
from base64 import b64encode, b64decode

class Cipher(object):
    def __init__(self, key):
        self.block_size = AES.block_size
        self.key = hashlib.sha512(key.encode()).digest()
        self.key = self.key[0:AES.block_size]

    def __pad(self, plain_text):
        number_of_bytes_to_pad = self.block_size - len(plain_text) % self.block_size
        ascii_string = chr(number_of_bytes_to_pad)
        padding_str = number_of_bytes_to_pad * ascii_string
        padded_plain_text = plain_text + padding_str
        return padded_plain_text

    @staticmethod
    def __unpad(plain_text):
        last_character = plain_text[len(plain_text) - 1:]
        bytes_to_remove = ord(last_character)
        return plain_text[:-bytes_to_remove]

    def encrypt(self, plain_text):
        plain_text = self.__pad(plain_text)
        iv = Random.new().read(self.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        encrypted_text = cipher.encrypt(plain_text.encode())
        return b64encode(iv + encrypted_text).decode("utf-8")

    def decrypt(self, encrypted_text):
        encrypted_text = b64decode(encrypted_text)
        iv = encrypted_text[:self.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        plain_text = cipher.decrypt(encrypted_text[self.block_size:]).decode("utf-8")
        return self.__unpad(plain_text)

    def get_version(self):
        return get_version()


class FileUtils(object):
    def __init__(self,key):
        self.cipher = Cipher(key)

    def encrypt_file(self, fpath, target):
        fr = open(fpath, 'r')
        fw = open(target, 'w')
        for line in fr.readlines():
            encrypted = self.cipher.encrypt(line)
            fw.write(encrypted)
            fw.write('\n')
        fr.close()
        fw.close()


    def decrypt_file(self, fpath, target):
        fr = open(fpath, 'r')
        fw = open(target, 'w')
        for line in fr.readlines():
            decrypted = self.cipher.decrypt(line)
            fw.write(decrypted)
        fr.close()
        fw.close()

    
def get_version():
    return 'Ver. 0.0.9'


from vtils import util
from vtils.util import Cipher

def test_version():
    assert util.get_version() == 'Ver. 0.0.9'


def test_security():
    text = "This is just for testing"
    key = "AbCdEf1234567890AbCdEf1234567890"
    cipher = Cipher(key)
    encrypted = cipher.encrypt(text)
    decrypted = cipher.decrypt(encrypted)
    assert decrypted == text


from setuptools import find_packages, setup
with open("README.md", "r") as fh:
    long_description = fh.read()
setup(
    name='vtils',
    packages=find_packages(include=['vtils']),
    version='0.1.0',
    description='Vtils Python Library',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    author='Venkateswara Rao Thota',
    license='MIT',
    install_requires=['PyCryptodome','boto3'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==4.4.1'],
    test_suite='tests',
)
